"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MachineImageService = void 0;
const node_util_1 = require("node:util");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const utility_1 = require("@jfrog-pipelines/utility");
const metadata_1 = require("@jfrog-pipelines/metadata");
const configuration_1 = require("@jfrog-pipelines/configuration");
const errors_1 = require("@jfrog-pipelines/errors");
const driven_1 = require("../../driven");
const MachineImageValidator_1 = require("./MachineImageValidator");
class MachineImageService {
    constructor() {
        this.buildPlaneImageDao = new driven_1.BuildPlaneImageDao();
        this.properties = configuration_1.ConfigFactory.config;
        this.masterIntegration = new metadata_1.MasterIntegrations();
    }
    validateUpsert(buildPlaneImage) {
        const masterIntegrations = this.masterIntegration.getMasterIntegrations(buildPlaneImage.provider);
        if (masterIntegrations.length === 0) {
            throw new errors_1.BadRequestError(`MasterIntegration not found for id: ${buildPlaneImage.provider}`);
        }
        const validateImage = {
            name: buildPlaneImage.name,
            masterIntegrationId: masterIntegrations[0].id,
            imageId: buildPlaneImage.imageId,
            region: buildPlaneImage.region,
            isAvailable: true,
            architecture: buildPlaneImage.architecture,
            operatingSystem: buildPlaneImage.os,
            metadataPropertyBag: buildPlaneImage.metadataPropertyBag
        };
        try {
            MachineImageValidator_1.MachineImageValidator.validateCreate(validateImage);
        }
        catch (error) {
            throw new errors_1.BadRequestError(`Image to upsert ${(0, node_util_1.inspect)(validateImage)} Error ${(0, node_util_1.inspect)(error)}`);
        }
        return {
            isInternal: true,
            ...validateImage
        };
    }
    async getBuildPlaneImageById(id, query) {
        let sequelizeQuery = {};
        if (query) {
            sequelizeQuery = this.constructGetQuery(query);
        }
        const imageAttributesModel = await this.buildPlaneImageDao.findById(id, sequelizeQuery);
        return imageAttributesModel;
    }
    async create(buildPlaneImageAttribute) {
        return this.buildPlaneImageDao.create(buildPlaneImageAttribute);
    }
    async getAllBuildPlaneImages(query) {
        const sequelizeQuery = this.constructGetQuery(query);
        const buildPlaneImages = await this.buildPlaneImageDao.findAll(sequelizeQuery);
        return buildPlaneImages;
    }
    constructGetQuery(query) {
        const sequelizeQuery = new data_access_layer_1.SequelizeQueryBuilder({}).attributes(query.includeAttributes);
        if (!(0, utility_1.isEmpty)(query.masterIntegrationIds)) {
            sequelizeQuery.where('masterIntegrationId', query.masterIntegrationIds);
        }
        if (query.isAvailable !== undefined) {
            sequelizeQuery.where('isAvailable', query.isAvailable);
        }
        const supportedOs = this.getSupportedOs();
        sequelizeQuery.where('operatingSystem', supportedOs);
        return sequelizeQuery.build();
    }
    getSupportedOs() {
        let supportedOs = (0, utility_1.keys)(metadata_1.NodeMetadata.supportedOsAndFamilyMap);
        if (!this.properties.isMacOsEnabled) {
            supportedOs = supportedOs.filter((os) => {
                return !os.startsWith('MacOS');
            });
        }
        if (this.properties.isMacOsEnabled && this.properties.isSaaS) {
            supportedOs = supportedOs.filter((os) => {
                return !(os.startsWith('MacOS') && !os.endsWith(this.properties.saasProps.saasEnvironment));
            });
        }
        return supportedOs;
    }
}
exports.MachineImageService = MachineImageService;
//# sourceMappingURL=MachineImageService.js.map