"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JobHandler = void 0;
const node_util_1 = require("node:util");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const metadata_1 = require("@jfrog-pipelines/metadata");
const logger_1 = require("@jfrog-pipelines/logger");
const microservice_1 = require("@jfrog-pipelines/microservice");
const utility_1 = require("@jfrog-pipelines/utility");
const BuildPlaneMessageDao_1 = require("../../driven/db/BuildPlaneMessageDao");
const CreateJobCommand_1 = require("../../driving/command/CreateJobCommand");
const NodeProvisioner_1 = require("../../domain/node/NodeProvisioner");
const acl_1 = require("../../driven/acl");
const NodeDomainService_1 = require("../../domain/node/NodeDomainService");
const ONE_MINUTE_MILLISECONDS = 60000;
const STALE_NODE_TIMEOUT = 3;
const WAITING_NODE_TIMEOUT = 15;
class JobHandler {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('JobHandler');
        const certInitializer = microservice_1.applicationContext.get(microservice_1.CERTIFICATE_INITIALIZER);
        this.buildPlaneMessageDao = new BuildPlaneMessageDao_1.BuildPlaneMessageDao();
        this.stepRepository = new acl_1.StepRepository();
        this.nodeDomainService = new NodeDomainService_1.NodeDomainService(certInitializer);
        this.nodeProvisioner = new NodeProvisioner_1.NodeProvisioner(certInitializer);
    }
    async createJobs(nodePool) {
        const buildPlaneMessages = await this.getBuildplaneMessages(nodePool);
        if (buildPlaneMessages.length < 0) {
            this.logger.info(`no buildplane message found for job nodepool ${nodePool.name}`);
            return;
        }
        for await (const buildPlaneMessage of buildPlaneMessages) {
            try {
                const stepToProcess = await this.getNextStep(buildPlaneMessage);
                if (!stepToProcess) {
                    this.logger.info(`Step in same affinity group already processing or no steps to process, skipping buildPlane message ${buildPlaneMessage.id}`);
                    await this.updateBuildPlaneMessage(buildPlaneMessage, buildPlaneMessage.nodeId);
                    continue;
                }
                const node = await this.getNode(buildPlaneMessage, nodePool);
                const jobCommand = await this.createJob(nodePool, stepToProcess, buildPlaneMessage);
                await this.assignJobToNode(node, jobCommand);
                await this.updateStepProperties(stepToProcess, node);
                await this.nodeDomainService.createJobInKubernetes(node);
                await this.updateBuildPlaneMessage(buildPlaneMessage, node.id);
            }
            catch (error) {
                this.logger.warn((0, node_util_1.inspect)(error));
            }
        }
    }
    async collectStaleJobs(nodePool) {
        const nodes = await this.nodeDomainService.getAllNodes(nodePool.id);
        if (nodes.length === 0) {
            return;
        }
        const nodesToCheck = nodes.filter((node) => node.statusCode === NodeDomainService_1.NodeStatus.waiting || node.statusCode === NodeDomainService_1.NodeStatus.success);
        const failedNodes = nodes.filter((node) => node.statusCode === NodeDomainService_1.NodeStatus.failure);
        const activeK8sJobs = await this.getActiveK8sJobs(nodePool, nodes[0]);
        const staleNodes = this.getStaleNodes(nodesToCheck, activeK8sJobs);
        const removeNodePromise = [];
        staleNodes.push(...failedNodes);
        for (const node of staleNodes) {
            removeNodePromise.push(this.nodeDomainService.removeNodeAndReference(node.id));
        }
        const staleWaitingNodes = this.getTimeoutNodes(nodesToCheck);
        for (const node of staleWaitingNodes) {
            removeNodePromise.push(this.nodeDomainService.applyJobToDeletedState(node));
        }
        await Promise.all(removeNodePromise);
        this.logger.info(`Collected ${removeNodePromise.length} jobs`);
    }
    async getBuildplaneMessages(nodePool) {
        const query = new data_access_layer_1.SequelizeQueryBuilder({}).where('nodePoolId', nodePool.id).build();
        return this.buildPlaneMessageDao.findAll(query);
    }
    async getNode(buildPlaneMessage, nodePool) {
        if (buildPlaneMessage.nodeId) {
            const node = await this.nodeDomainService.getNode(buildPlaneMessage.nodeId, [nodePool.id]);
            if (node) {
                return node;
            }
        }
        return this.nodeProvisioner.provisionBeta(nodePool);
    }
    async getNextStep(buildPlaneMessage) {
        if (buildPlaneMessage.statusCode === metadata_1.READY) {
            return this.getNextStepForReadyBuildPlaneMessage(buildPlaneMessage);
        }
        return this.getNextStepForProcessingBuildPlaneMessage(buildPlaneMessage);
    }
    getNextStepForReadyBuildPlaneMessage(buildPlaneMessage) {
        const steps = buildPlaneMessage.payload.steps;
        if (steps.length === 0) {
            return;
        }
        return steps[0];
    }
    async getNextStepForProcessingBuildPlaneMessage(buildPlaneMessage) {
        const steps = buildPlaneMessage.payload.steps;
        const stepIds = buildPlaneMessage.payload.stepIds;
        if (steps.length === 0) {
            return;
        }
        const step = await this.stepRepository.getStepById(steps[0].stepId);
        const stepStatus = step.statusCode;
        if (!metadata_1.TERMINAL_STEP_STATUS_BY_CODE.includes(stepStatus)) {
            return;
        }
        const node = await this.nodeDomainService.getNode(buildPlaneMessage.nodeId, [buildPlaneMessage.nodePoolId]);
        await this.nodeDomainService.removeJobsForStepId(step.id, node);
        steps.shift();
        stepIds.shift();
        await this.buildPlaneMessageDao.updateByInstance(buildPlaneMessage, {
            payload: {
                ...buildPlaneMessage.payload,
                stepIds: stepIds,
                steps: steps
            }
        });
        return steps.at(0);
    }
    async createJob(nodePool, step, buildPlaneMessage) {
        const nodePoolIntegrations = this.getIntegrationsFromNodePool(nodePool);
        const steplets = await this.getSteplets(step.stepletIds);
        const stepletIntegrations = this.getIntegrationsFromSteplets(steplets);
        const integrationsSet = new Set([...nodePoolIntegrations, ...stepletIntegrations]);
        const integrations = [...integrationsSet.values()];
        const jobCommand = this.createJobCommand(nodePool, step.stepId, steplets, buildPlaneMessage, integrations);
        return jobCommand;
    }
    getIntegrationsFromNodePool(nodePool) {
        const customTags = nodePool.providerMetadataPropertyBag.customTags;
        if (!(0, utility_1.isEmpty)(customTags)) {
            for (const customTag of customTags) {
                if (customTag.Key === 'customSecrets') {
                    return customTag.Value.split(',');
                }
            }
        }
        return [];
    }
    getIntegrationsFromSteplets(steplets) {
        const stepletIntegrations = [];
        for (const steplet of steplets) {
            const configPropertyBag = steplet.configPropertyBag;
            if (configPropertyBag?.registry) {
                stepletIntegrations.push(configPropertyBag.registry);
            }
        }
        return stepletIntegrations;
    }
    async assignJobToNode(node, jobCommand) {
        const job = jobCommand.getJob();
        const updatedNode = await this.nodeDomainService.updateNode(node, {
            systemPropertyBag: {
                ...node.systemPropertyBag,
                job: job
            },
            instanceId: node.friendlyName
        });
        this.logger.info(`Processed job for step ${job.stepId}`);
        return updatedNode;
    }
    async updateStepProperties(step, node) {
        const originalStep = await this.stepRepository.getStepById(step.stepId);
        await this.updateStepConfigPropertyBag(originalStep, node);
    }
    async updateBuildPlaneMessage(buildPlaneMessage, nodeId) {
        const steps = buildPlaneMessage.payload.steps;
        if (steps.length === 0) {
            const deleteQuery = new data_access_layer_1.SequelizeQueryBuilder({}).where('id', buildPlaneMessage.id).build();
            await this.buildPlaneMessageDao.delete(deleteQuery);
            return;
        }
        await this.buildPlaneMessageDao.updateByInstance(buildPlaneMessage, {
            statusCode: metadata_1.PROCESSING,
            nodeId: nodeId
        });
    }
    async updateStepConfigPropertyBag(step, node) {
        const existingNodeIds = step.configPropertyBag.jobNodeIds ?? [];
        const configPropertyBag = {
            ...step.configPropertyBag,
            isJob: true,
            nodeId: node.id,
            jobNodeIds: [...existingNodeIds, node.id]
        };
        return this.stepRepository.updateStepConfigPropertyBag(configPropertyBag, step.id);
    }
    createJobCommand(nodePool, stepId, steplets, buildPlaneMessage, stepIntegrations) {
        const stepletInfos = this.getStepletInfos(steplets);
        const job = {
            nodePoolId: nodePool.id,
            stepId: stepId,
            stepletInfo: stepletInfos,
            runId: buildPlaneMessage.runId,
            affinityGroup: buildPlaneMessage.payload.affinityGroup,
            builderApiToken: buildPlaneMessage.payload.builderApiToken,
            builderSetupApiToken: buildPlaneMessage.payload.builderSetupApiToken,
            localArtifactoryToken: buildPlaneMessage.payload.localArtifactoryToken,
            stepIntegrations: stepIntegrations
        };
        return CreateJobCommand_1.CreateJobCommand.create(job);
    }
    async getSteplets(stepletIds) {
        const stepletPromises = [];
        for (const stepletId of stepletIds) {
            stepletPromises.push(this.stepRepository.getStepletById(stepletId));
        }
        return Promise.all(stepletPromises);
    }
    getStepletInfos(steplets) {
        const stepletInfos = [];
        for (const steplet of steplets) {
            const defaultStepletRuntimeImageName = steplet.configPropertyBag?.imageName;
            const defaultStepletRuntimeImageTag = steplet.configPropertyBag?.imageTag;
            const matrixStepletRuntime = steplet.configPropertyBag?.runtime;
            const matrixStepletRuntimeImage = matrixStepletRuntime?.image;
            const matrixStepletRuntimeImageName = matrixStepletRuntimeImage?.imageName;
            const matrixStepletRuntimeImageTag = matrixStepletRuntimeImage?.imageTag;
            const stepletDetails = {
                stepletId: steplet.id,
                image: (defaultStepletRuntimeImageName ?? matrixStepletRuntimeImageName),
                tag: (defaultStepletRuntimeImageTag ?? matrixStepletRuntimeImageTag)
            };
            stepletInfos.push(stepletDetails);
        }
        return stepletInfos;
    }
    async getActiveK8sJobs(nodePool, node) {
        const namespace = (nodePool.providerMetadataPropertyBag.namespace ?? 'default');
        return this.nodeDomainService.getJobs(namespace, node);
    }
    getStaleNodes(nodes, jobs) {
        return nodes.filter((node) => {
            const lastNodeUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
            const CUTOFF_TIME = STALE_NODE_TIMEOUT * ONE_MINUTE_MILLISECONDS;
            const cutOffTimeForWaitingAt = Date.now() - CUTOFF_TIME;
            return !jobs.includes(`${node.friendlyName}-deployment`) && lastNodeUpdatedAt < cutOffTimeForWaitingAt;
        });
    }
    getTimeoutNodes(nodes) {
        return nodes.filter((node) => {
            const lastNodeUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
            const CUTOFF_TIME = WAITING_NODE_TIMEOUT * ONE_MINUTE_MILLISECONDS;
            const cutOffTimeForWaitingAt = Date.now() - CUTOFF_TIME;
            return lastNodeUpdatedAt < cutOffTimeForWaitingAt;
        });
    }
}
exports.JobHandler = JobHandler;
//# sourceMappingURL=JobHandler.js.map