"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessNodePoolHandler = exports.LOCK_TIMEOUT_IN_MS = void 0;
const node_util_1 = require("node:util");
const microservice_1 = require("@jfrog-pipelines/microservice");
const logger_1 = require("@jfrog-pipelines/logger");
const NodeCollector_1 = require("../../domain/node/NodeCollector");
const NodeProvisioner_1 = require("../../domain/node/NodeProvisioner");
const NodePoolService_1 = require("../../domain/nodepool/NodePoolService");
const FeatureAwareFactory_1 = require("../../shared/FeatureAwareFactory");
const JobHandler_1 = require("./JobHandler");
exports.LOCK_TIMEOUT_IN_MS = 60000;
class ProcessNodePoolHandler {
    constructor(command, serviceMetaData) {
        this.command = command;
        this.serviceMetaData = serviceMetaData;
        this.logger = logger_1.LoggerFactory.getLogger('nodePoolWorker');
        this.nodePoolService = new NodePoolService_1.NodePoolService();
    }
    async handle() {
        const poolID = this.command.getNodePoolId();
        const systemInstanceId = microservice_1.applicationContext.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER)
            ?.systemServiceInstancesId;
        this.logger.trace(`Processing for node pool with id: ${poolID} with system service id: ${systemInstanceId}`);
        const lockName = [this.serviceMetaData.name, '-nodePools-', ...String(poolID)].join('');
        const lock = await microservice_1.Lock.create(systemInstanceId, lockName, exports.LOCK_TIMEOUT_IN_MS);
        if (lock === null) {
            return;
        }
        try {
            const onDemandNodeStartTime = Date.now();
            const nodePool = await this.nodePoolService.getNodePool(poolID);
            if (!nodePool) {
                this.logger.warn(`NodePool id ${poolID} is not dynamic nodepool`);
                return;
            }
            this.logger.trace(`Successfully fetched node pool with id: ${poolID} and node pool name is ${nodePool.name}`);
            if (!nodePool.isOnDemand) {
                this.logger.info(`Node Pool is not on demand, wont continue with execution`);
                return;
            }
            if (this.isNodePoolJobType(nodePool)) {
                this.logger.info('Processing job creation request', (0, node_util_1.inspect)(this.command));
                const jobHandler = new JobHandler_1.JobHandler();
                await jobHandler.createJobs(nodePool);
                await jobHandler.collectStaleJobs(nodePool);
                return;
            }
            const certInitializer = microservice_1.applicationContext.get(microservice_1.CERTIFICATE_INITIALIZER);
            const collector = new NodeCollector_1.NodeCollector();
            const provisioner = new NodeProvisioner_1.NodeProvisioner(certInitializer);
            await lock.obtainLock();
            this.logger.trace(`Successfully obtained lock for node pool ${nodePool.name} with lock name: ${lockName}`);
            const onDemandNodeEndTime = Date.now();
            const onDemandNodeTimeDiff = new Date(onDemandNodeEndTime - onDemandNodeStartTime);
            const requiredNodes = await this.getRequiredNodes(poolID);
            const onDemandNodeData = {
                domain: 'nodepool',
                eventType: 'onDemandNodes',
                durationMillis: onDemandNodeTimeDiff.valueOf(),
                count: requiredNodes,
                nodePoolName: nodePool.name
            };
            logger_1.LoggerFactory.logEvent(onDemandNodeData);
            this.logger.trace(`Required Node count is ${requiredNodes}`);
            if (nodePool.isBeta) {
                this.logger.trace('Beta node pool, will not de-provision node');
            }
            else {
                await provisioner.provision(nodePool, requiredNodes);
                await collector.deProvision(nodePool, requiredNodes);
                await this.collectOrphanNodes(collector);
            }
        }
        catch (error) {
            this.logger.error(`Failed to process node pool id ${poolID} with err ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            await lock.releaseLock();
        }
    }
    isNodePoolJobType(nodePool) {
        if (!Array.isArray(nodePool.providerMetadataPropertyBag.customTags)) {
            return false;
        }
        const customTags = nodePool.providerMetadataPropertyBag?.customTags;
        for (const tag of customTags) {
            if (tag.Key === 'executionType' && tag.Value === 'job') {
                return true;
            }
        }
        return false;
    }
    async getRequiredNodes(poolID) {
        const features = new FeatureAwareFactory_1.FeatureAwareFactory();
        return features.getRequiredNodes(poolID);
    }
    async collectOrphanNodes(collector) {
        this.logger.debug('Collecting orphan nodes started');
        const collectedNodeCount = await collector.collectJunkNodes();
        this.logger.debug(`${collectedNodeCount} orphan nodes were successfully collected`);
    }
}
exports.ProcessNodePoolHandler = ProcessNodePoolHandler;
//# sourceMappingURL=ProcessNodePoolHandler.js.map